/*
 * Copyright (C) 2014
 *      Andrew Hayzen <ahayzen@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.3

Item {
    property Ball ball
    property Character character
    property var gravity
    property int maxX
    property int minX

    // For a given diff scale the movement
    function getScalarForDiff(diff) {
        var scalar;

        if (diff > units.gu(4)) {
            scalar = character.scalarLimit
        } else if (diff > units.gu(2)) {
            scalar = character.scalarLimit / 2
        } else {
            scalar = character.scalarLimit / 4
        }

        return scalar;
    }

    function iter() {
        var u = ball.linearVelocity  // initial velocity
        var t = 0;
        var sx = 0;
        var topY = gameScene.height - ball.y;

        if (u.y < 0) {  // ball going up
            // calculate the diff to the Y top
            // s = (v^2 - u^2) / (2 * a)
            var diffY = (0 - u.y * u.y) / (2 * gravity.y);

            topY -= diffY * gameScene.pixelsPerMeter;

            // calculate the diff in x from the time to top of Y
            t = diffY / u.y;
            sx += u.x * t;
        } else {  // ball going down so top is current
            ;
        }

        // time until the ball touches the top of the character (character.radius)
        // v^2 = u^2 + 2*a*s
        // s = 1/2 (u + v) * t
        // t = s / (1/2 (u + root(u + 2 *a*s)))
        t = ((topY - character.radius) / gameScene.pixelsPerMeter) / (0.5 * Math.sqrt(0 + 2 * gravity.y * ((topY - character.radius) / gameScene.pixelsPerMeter)))

        // calculate the X distance in the remaining arc
        sx += u.x * t;

        // Predicted x at y=top of character
        var predX = ball.x + sx * gameScene.pixelsPerMeter;

        // Check not bouncing off a wall
        if (predX > maxX) {
            predX = maxX - (predX - maxX)
        } else if (predX < 0) {
            predX = -predX
        }

        // Debug UI
        aiPred.x = predX;
        aiPred.y = gameScene.height - wallDepth;

        aiPredX.x = predX;
        aiPredY.y = gameScene.height - topY;

        // Decide based on known information actions to perform
        // TODO: if in the front half move more right to hit it over
        // TODO: take angle into account

        if (Math.abs(ball.x - character.center) < character.radius &&
                character.y + character.radius > ball.y &&
                character.y - ball.y < units.gu(10)) {
            // Ball is within the radius of the character and less that X units above the char
            if (ball.x >= character.center - units.gu(.5)) {
                // Ball is to the right of the center
                character.right()
            }

            character.jump()
        } else if (predX < minX && ball.x < minX) {
            // Ball is out of court and predicted to land there
            var center = minX + (maxX - minX) / 2;

            // Attempt to return to the center-front
            if (Math.abs(character.center - (center - character.radius)) > units.gu(2)) {
                if (character.center > center - character.radius) {
                    character.left(getScalarForDiff(character.center - center - character.radius))
                } else {
                    character.right(getScalarForDiff(center - character.radius - character.center))
                }
            }
        } else if (predX < character.center - units.gu(1) &&
                   Math.abs(character.x - minX) > units.gu(.5)) {
            // Predicted landing is left and char is not against left wall
            character.left(getScalarForDiff(character.center - units.gu(1) - predX))
        } else if (predX > character.center + units.gu(1) &&
                   Math.abs((character.center + character.radius) - maxX) > units.gu(.5)) {
            // Predicted landing is right and char is not against right wall
            character.right(getScalarForDiff(predX - character.center + units.gu(1)))
        }
    }
}
